﻿/*
 3D sensor Display tool
 Copyright Jean Louis Coural  03/2020

  https://docs.microsoft.com/fr-fr/dotnet/framework/wpf/graphics-multimedia/how-to-rotate-an-object
  https://www.codeproject.com/Articles/130109/Serial-Communication-using-WPF-RS232-and-PIC-Commu
*/


using HelixToolkit.Wpf;
using System;
using System.Windows;
using System.Windows.Shapes;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Media3D;
using System.IO.Ports; //used for scanning the serial ports
using System.Windows.Threading;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Globalization;

delegate void StringArgReturningVoidDelegate(string text); //used for fixing multithreading problems

namespace Display3DModel
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        //Path to the model file
        //private const string MODEL_PATH = "3DModel/File01.obj";
        private const string MODEL_PATH = "Model/model.stl";
        private double[] dataz = {0.0, 0.0, 0.0, 0.0};
        public ModelVisual3D device3D = new ModelVisual3D();
        public Vector3D Xaxis = new Vector3D(1, 0, 0);
        public Vector3D Yaxis = new Vector3D(0, 1, 0);
        public Vector3D Zaxis = new Vector3D(0, 0, 1);
        public double angle = 0;
        public Matrix3D matrix;
        public Point3D center;
        public Model3D torp;
        public SerialPort serialPort1 = new SerialPort();
        private delegate void UpdateUiTextDelegate(char[] text);

        public MainWindow()
        {
            InitializeComponent();
            serialPort1.DataReceived += new System.IO.Ports.SerialDataReceivedEventHandler(serialPort1_DataReceived);

            //dataz = new double[5];
            portComboBox.SelectedIndex = 0;
            RateComboBox.SelectedIndex = 1;
            //viewPort3d.ShowCoordinateSystem = true;
            viewPort3d.ShowFieldOfView = true;

            center = new Point3D(0, 0, 0);
            //Point3D Ztop = new Point3D(0, 0, 30);
            //CreateLineSegment(center, Ztop);
            //device3D.Children.Add(new GridLinesVisual3D() { Center = center, Normal = Zaxis, Width = 150, Length = 100, MinorDistance = 10, MajorDistance = 10, Thickness = 0.1, Fill = Brushes.Green });

            torp = Display3d(MODEL_PATH);

            device3D.Content = torp;

            // Add to view port
            viewPort3d.Children.Add(device3D);

            // gravity center on origin
            Yaxis.Y = -40;
            matrix.Translate(Yaxis);
            device3D.Transform = new MatrixTransform3D(matrix);
            Yaxis.Y = 1;

        }

        private void serialPort1_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            SerialPort sp = (SerialPort)sender;            
            char[] received_data = new char[512];

            System.Threading.Thread.Sleep(100);            
            try
            {
                sp.Read(received_data, 0, 512);
                if (!Dispatcher.CheckAccess())
                    Dispatcher.Invoke(DispatcherPriority.Background, new UpdateUiTextDelegate(WriteData), received_data);
                else
                    WriteData(received_data);
            }
            catch (Exception ex) { }

        }

    

        private void WriteData(char[] buf)
        {
            string output = "";
            string s = new string(buf);
            var splits = s.Split(new string[] { "START", "STOP" }, StringSplitOptions.None);
            if (splits.Length >= 3)
            {
                if (splits[1] != "")
                    output = splits[1];
                else
                    output = splits[2];
            }
            if (output != "")
            {
                textBox2.Text = output;
                Move_3DO(output);
                //System.Threading.Thread.Sleep(300); pas ici !
            }
        }


        // move 3D image
        private void Move_3DO(string buf)
        {
            if ((bool)Yaw_RadioButton.IsChecked)
            {
                var splits2 = buf.Split(new string[] { "AZIMUTH: ", "\r\n" }, StringSplitOptions.None);
                try
                {
                    if (!Double.TryParse(splits2[2], NumberStyles.Number, CultureInfo.InvariantCulture, out double az)) return;
                    az = MovingAverage(az);
                    az = az > 180 ? az - 360 : az;
                    az -= angle;
                    angle += az;
                    matrix.Rotate(new Quaternion(Zaxis, az));
                    device3D.Transform = new MatrixTransform3D(matrix);
                }
                catch (Exception ex) { }
            }
            else  // pitch
            {
                var splits2 = buf.Split(new string[] { "PITCH: ", "\r\n" }, StringSplitOptions.None);
                try
                {
                    if (!Double.TryParse(splits2[1], NumberStyles.Number, CultureInfo.InvariantCulture, out double az)) return;
                    az = MovingAverage(az);                   
                    az -= angle;
                    angle += az;
                    matrix.Rotate(new Quaternion(Xaxis, az));
                    device3D.Transform = new MatrixTransform3D(matrix);
                }
                catch (Exception ex) { }
            }

        }

        /// <summary>
        /// Display 3D Model
        /// </summary>
        /// <param name="model">Path to the Model file</param>
        /// <returns>3D Model Content</returns>
        private Model3D Display3d(string path)
        {
            Model3DGroup devices = null;
            Model3D device = null;

            try
            {
                //Adding a gesture here
                viewPort3d.RotateGesture = new MouseGesture(MouseAction.LeftClick);

                //Import 3D model file
                ModelImporter importer = new ModelImporter();

                // Load the 3D model file
                devices = importer.Load(path);
                device = devices.Children[0];
                GeometryModel3D model = device as GeometryModel3D;
                DiffuseMaterial material = new DiffuseMaterial(new SolidColorBrush(Colors.DarkGray));
                model.Material = material;

            }
            catch (Exception e)
            {
                // Handle exception in case can not file 3D model
                MessageBox.Show("Exception Error : " + e.StackTrace);
            }
            return device;
        }



        private void move_right_click(object sender, RoutedEventArgs e)
        {  // https://stackoverrun.com/fr/q/10045501//var axis = new Vector3D(0, 0, 1);
            angle = 10;
            if ((bool)Yaw_RadioButton.IsChecked)
                matrix.Rotate(new Quaternion(Zaxis, angle));
            else
                matrix.Rotate(new Quaternion(Xaxis, angle));
            device3D.Transform = new MatrixTransform3D(matrix);
        }

        private void move_left_click(object sender, RoutedEventArgs e)
        {
            angle = -10;
            if ((bool)Yaw_RadioButton.IsChecked)
                matrix.Rotate(new Quaternion(Zaxis, angle));
            else
                matrix.Rotate(new Quaternion(Xaxis, angle));
            device3D.Transform = new MatrixTransform3D(matrix);
        }

        private void X_click(object sender, RoutedEventArgs e)
        {            
            if (!Int32.TryParse(Xbox.Text, out int x)) return;
            Xaxis.X = x;
            matrix.Translate(Xaxis);
            device3D.Transform = new MatrixTransform3D(matrix);
        }

        private void Y_click(object sender, RoutedEventArgs e)
        {           
            if (!Int32.TryParse(Ybox.Text, out int y)) return;
            Yaxis.Y = y;
            matrix.Translate(Yaxis);
            device3D.Transform = new MatrixTransform3D(matrix);
        }

        private void Connect_Click(object sender, RoutedEventArgs e)
        {
            if (serialPort1.IsOpen) return;
            try
            {
                if (!Int32.TryParse(RateComboBox.Text, out int br)) br = 9600;
                serialPort1.BaudRate = br;
                serialPort1.PortName = portComboBox.Text;
                serialPort1.Open(); //the specific command to open the serial port
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);//displaying the error in a message box
            }
        }



        private void Scan_Click(object sender, RoutedEventArgs e)
        {
            int itemsFound = 0; //how many active ports are found

            //finding available serial ports and filling up the combo box with found port names
            foreach (string s in SerialPort.GetPortNames())
            {
                if(portComboBox.Items.IndexOf(s) == -1 ) portComboBox.Items.Add(s);//adding port name to combo box
                itemsFound++;//incrementing the items found variable
            }

            if (itemsFound < 1)
                MessageBox.Show("NO ITEMS FOUND");//if no serial devices have been found, an error message window will show up
            else
            {
                int index = portComboBox.Items.IndexOf("COM8");  // Arduino serial port
                if (index != -1)
                    portComboBox.SelectedIndex = index;  //if there were found active ports
                else
                    portComboBox.SelectedIndex = 0;
            }
        }

        private void Disconnect_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (serialPort1.IsOpen)
                {
                    Thread CloseDown = new Thread(new ThreadStart(serialPort1.Dispose));
                    CloseDown.Start();
                    while (serialPort1.IsOpen);
                    CloseDown.Abort();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);//displaying the error in a message box
            }
        }

        private void ON_MAINW_CLOSING(object sender, CancelEventArgs e)
        {
            if (serialPort1.IsOpen)
            {
                //serialPort1.Close(); //fait tout planter: executer dans un autre thread
                Thread CloseDown = new Thread(new ThreadStart(serialPort1.Dispose));
                CloseDown.Start();
                while (serialPort1.IsOpen) ;
                CloseDown.Abort();
            }
        }


        private void portComboBox_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            serialPort1.PortName = portComboBox.Text;
        }

        private void RateComboBox_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            if (!Int32.TryParse(RateComboBox.Text, out int br)) return;
            serialPort1.BaudRate = Convert.ToInt16(RateComboBox.Text);
        }


        private void CreateLineSegment(Point3D A, Point3D B)
        {
            LinesVisual3D line = new LinesVisual3D();
            line.Color = Colors.Black;
            line.Thickness = 1;
            line.Points.Add(A);
            line.Points.Add(B);
            viewPort3d.Children.Add(line);

        }

        double MovingAverage(double val)
        {
            double result = 0;
            for(int i=dataz.Length-1;i>=0 ; i--)
            {
                if (i > 0) dataz[i] = dataz[i - 1];
                else dataz[i] = val;
                result += dataz[i];
            }

            result /= dataz.Length;
            return result;
        }

        private void test_click(object sender, RoutedEventArgs e)
        {
            if (!Double.TryParse(Xbox.Text, NumberStyles.Number, CultureInfo.InvariantCulture, out double val)) return;
            Test_TextBox.Text = MovingAverage(val).ToString();
        }
    }

}